c  hde1.f
c  1-D heat diffusion equation explicit solver.
c  Miroslav Broz (miroslav.broz@email.cz), Apr 25th 2007

      program hde1

      include 'const.inc'
c
c  variables
c

c  max dimension of the 1-D spatial grid
      integer Nmax
      parameter(Nmax=10000)

      real*8 Temp(0:Nmax),Temp_new(0:Nmax),x(0:Nmax)
      real*8 t,tmin,tmax,dt,dtout,tout,dx,xi,K,C,rho,D
      real*8 A_Bond,epsil,r,T_eq,T_star,P,f,delta
      real*8 n_perp(3),n_Sun_t0(3),E
      integer N,j
      character*80 s

c  functions
      real*8 Insolation,sgn

c
c  read input parameters
c
      write(*,15) '# rho [kg/m^3] = '
15    format(a,$)

5     continue
        read(*,*,err=990,end=990) s
      if (s(1:1).eq.'#') goto 5

      read(s,*,err=990,end=990) rho
      write(*,*) rho

      write(*,15) '# C [J/kg/K] = '
      read(*,*,err=990,end=990) C
      write(*,*) C

      write(*,15) '# K [W/m/K] = '
      read(*,*,err=990,end=990) K
      write(*,*) K

      write(*,15) '# A_Bond [] = '
      read(*,*,err=990,end=990) A_Bond
      write(*,*) A_Bond

      write(*,15) '# epsilon [] = '
      read(*,*,err=990,end=990) epsil
      write(*,*) epsil

      write(*,15) '# P [h] = '
      read(*,*,err=990,end=990) P
      P = P*3600.d0
      write(*,*) P/3600.d0

      write(*,15) '# r [AU] = '
      read(*,*,err=990,end=990) r
      write(*,*) r

      write(*,15) '# n_perp [x,y,z] = '
      read(*,*,err=990,end=990) n_perp(1),n_perp(2),n_perp(3)
      call normalize(n_perp)
      write(*,*) n_perp(1),n_perp(2),n_perp(3)

      write(*,15) '# n_Sun_t0 [x,y,z] = '
      read(*,*,err=990,end=990) n_Sun_t0(1),n_Sun_t0(2),n_Sun_t0(3)
      call normalize(n_Sun_t0)
      write(*,*) n_Sun_t0(1),n_Sun_t0(2),n_Sun_t0(3)

      write(*,30) '# N [] (Nmax = ',Nmax,') = '
30    format(a,i5,a,$)
      read(*,*,err=990,end=990) N
      write(*,*) N

      write(*,15) '# dx [m] = '
      read(*,*,err=990,end=990) dx
      write(*,*) dx

      write(*,15) '# dt [s] = '
      read(*,*,err=990,end=990) dt
      write(*,*) dt

      write(*,15) '# dtout [s] = '
      read(*,*,err=990,end=990) dtout
      write(*,*) dtout

      write(*,15) '# tmin [h] = '
      read(*,*,err=990,end=990) tmin
      tmin = tmin*3600.d0
      write(*,*) tmin/3600.d0

      write(*,15) '# tmax [h] = '
      read(*,*,err=990,end=990) tmax
      tmax = tmax*3600.d0
      write(*,*) tmax/3600.d0

      goto 995
990   continue
        write(10,991) '# Error: reading input parameters.'
991     format(/,a)
        stop
995   continue

c
c  pre-calculate some parameters
c

c  diffusivity
      D = K/(rho*C)
      write(*,20) '# D [m^2/s] = ', D

c  von Neumann stability criterion
      xi = 2*D*dt/(dx**2)
      write(*,20) '# xi [] = ', xi
20    format(a,f10.5)

      if (xi.gt.1) then
        write(*,10) '# Warning: von Neumann stability criterion is NOT'
     :    // ' fulfilled!'
10      format(a)
      endif

c  equilibrium temperature
c  pi R^2 (1-A) L_Sun / (4 pi r^2) = 4 pi R^2 epsilon sigma T_eq^4
      T_eq = ((1-A_Bond)*S0 / (r**2 * 4*epsil*sigma))**0.25d0
      write(*,20) '# T_eq [K] = ', T_eq

c  subsolar temperature
      T_star = sqrt(2.d0)*T_eq
      write(*,20) '# T_star [K] = ', T_star

c  diurnal/seasonal frequency
      f = 1.d0/P
      write(*,20) '# f [Hz] = ', f
      write(*,20) '# f [rev/day] = ', day/P

c  penetration depth
      delta = sqrt(D/(pi*f))
      write(*,20) '# delta [m] = ', sgn(dx)*delta

c  spatial coordinates
      do j = 0, N
        x(j) = j*dx
      enddo
c
c  initial conditions (t=0)
c
      do j = 0, N
        Temp(j) = T_eq
      enddo
      Temp(0) = T_star
c
c  an initial dump
c
      write(*,10) '# t [s] & x [m] & T [K]'
      t = tmin
      tout = tmin + dtout
      write(*,20) '# t [s] = ', t
      do j = 0, N
        write(*,*) t, x(j), Temp(j)
      enddo
      write(*,*)
c
c  main loop
c
      do while (t.le.(tmax - 0.5d0*dt))

c  boundary condition (j=0)
c  K (dT/dr)_surface + epsilon sigma T^4 = (1-A) * E(t) * n_perp(r)

c  for K=0 it is simple...
        E = insolation(t,f,r,n_perp,n_Sun_t0)
        Temp(0) = ((1.d0-A_Bond) * E / (epsil*sigma))**0.25d0

c  boundary condition (j=N)
c  dT/dr = 0
        Temp(N) = Temp(N-1)

        call ftcs_step(Temp,N,dt,dx,D,Temp_new)

        t = t + dt
        do j = 0, N
          Temp(j) = Temp_new(j)
        enddo

c  print output every dtout
        if (t.ge.(tout-0.5d0*dt)) then
          write(*,20) '# t [s] = ', t
          do j = 0, N
            write(*,*) t, x(j), Temp(j)
          enddo
          write(*,*)
          tout = tout + dtout
        endif

      enddo

      stop
      end


